//
// Sample code for calling A.nnotate api from Java.
//
// Edit the values in main() to include your server's
// admin email, api key and sample document date/code.
//
// For a sample JSON library in Java:
//    http://www.json.org/java/
//

import sun.misc.BASE64Encoder;
import javax.crypto.Mac;
import javax.crypto.spec.SecretKeySpec;
import java.net.URLEncoder;
import java.util.*;
import java.net.URL;
import java.net.URLConnection;
import java.io.*;

import org.json.*;

public class AnnotateApi {

    // Make a base64 encoded HMAC / SHA1 hash of a string.
    public static String sign(String apikey, String data) throws Exception {
        Mac mac = Mac.getInstance("HmacSHA1");
        byte[] keyBytes = apikey.getBytes("UTF8");
        SecretKeySpec signingKey = new SecretKeySpec(keyBytes, "HmacSHA1");
        mac.init(signingKey);

        byte[] signBytes = mac.doFinal(data.getBytes("UTF8"));
        String signature = (new BASE64Encoder()).encode(signBytes).replaceAll("\n","");
        return signature; 
    }

    //
    // Create a signed a.nnotate api request GET parameters
    // (expires in a few minutes)
    //
    public static String signRequest( String phpfn, 
                                      String apiuser, 
                                      String apikey, 
                                      String annotateuser ) throws Exception {
        return signRequest(phpfn, apiuser, apikey, annotateuser, 0 );
    }

    //
    // Create a signed a.nnotate api request (validfor given number of seconds)
    //
    public static String signRequest( String phpfn, 
                                      String apiuser, 
                                      String apikey, 
                                      String annotateuser,
                                      long validfor)  throws Exception {        
        long requesttime = (new Date()).getTime() + validfor;
   
        String stringToSign = phpfn + "\n" + apiuser+ "\n"+ requesttime + "\n" + annotateuser;
        String signature = sign(apikey, stringToSign);
        
        return "api-user=" + URLEncoder.encode(apiuser, "UTF-8") + 
            "&api-requesttime=" + URLEncoder.encode(""+requesttime, "UTF-8") + 
            "&api-annotateuser=" + URLEncoder.encode(annotateuser, "UTF-8") + 
            "&api-auth=" + URLEncoder.encode(signature, "UTF-8");
    }
    

    // Do a HTTP GET of a url.
    public static String fetchURL(String loc) throws Exception {
        StringBuffer ret = new StringBuffer();
        URL u = new URL(loc);

        BufferedReader br = new BufferedReader( new InputStreamReader(u.openStream()));
        String line = "";
        while ((line=br.readLine()) != null) {
            ret.append(line);
        }
        return ret.toString();
    }

    // Do an HTTP POST to a url.
    // loc: the URL (with GET params)
    // contents: the POST params
    public static String postURL(String loc, String content) throws Exception {
        StringBuffer ret = new StringBuffer();
        URL u = new URL(loc);
        URLConnection conn = u.openConnection();

        conn.setDoInput (true);
        conn.setDoOutput (true);
        conn.setUseCaches (false);

        conn.setRequestProperty("Content-Type", "application/x-www-form-urlencoded");

        DataOutputStream out = new DataOutputStream (conn.getOutputStream());
        out.writeBytes(content);
        out.flush();
        out.close();

        BufferedReader in = new BufferedReader (new InputStreamReader(conn.getInputStream()));
        String temp;
        while ((temp = in.readLine()) != null){
            ret.append(temp);
        }
        return ret.toString();
    }    

    // Encode a string without raising java exceptions
    public static String urlencode(String s) {
        try {
            return URLEncoder.encode(s, "UTF-8");
        }
        catch (Exception e) {
            return "" + e;
        }
    }

    // Code name value map to form params (p1=abc&p1=xyz), urlencoding the values
    public static String urlencodeMap(HashMap map) {
        Iterator iter = map.keySet().iterator();
        StringBuffer ret = new StringBuffer();
        while (iter.hasNext()) {
            Object key = iter.next();
            Object val = map.get(key);
            String sval = urlencode(""+val);
            String skey = ""+key;
            if (ret.length() > 0) { ret.append("&"); }
            ret.append(skey);
            ret.append("=");
            ret.append(sval);
        }           
        return ret.toString();
    }


    // Parse / display the return value from an API call.
    public static void parseReturnValue(String ret) throws Exception {
        // Deal with return value:
        if (ret.substring(0, 2).equals("OK")) {
            System.out.println("Success - returned: " + ret);
        }        
        else if (ret.substring(0, 3).equals("ERR")) {
            System.out.println("Error - returned: " + ret);
        }
        else if (ret.substring(0,1).equals("{")) {
            JSONObject jobj = new JSONObject( ret );
            System.out.println("Parsed JSON Object return value as:\n" + jobj.toString(4));            
        }
        else if (ret.substring(0,1).equals("[")) {
            JSONArray jary = new JSONArray( ret );
            System.out.println("Parsed JSON Array return value as:\n" + jary.toString(4));
        }   
    }


    //
    // Try calling an A.nnotate server.
    //
    public static void main(String[] args) throws Exception {

        // *** Replace these settings with your apikey / apiuser    
        String apikey = "5bgNp5Ie2EclyX8gS8DIrUxSlExHflrA";
        String apiuser = "admin@example.com";
        String annotateuser = "joe@example.com"; 
        String annotateServer = "http://localhost/annotate"; 
        String ws = "112633952";

        // Example 1: Calling an API function which uses HTTP GET ===
        String phpfn = "listDocuments.php";        
        // String phpfn = "listUsers.php";        
        String request = signRequest(phpfn, apiuser, apikey, annotateuser, 0);
        request += "&ws=" + ws;
        System.out.println( "\n\nExample 1 - HTTP GET of API method " + request );

        // Do the HTTP GET
        String loc = annotateServer + "/php/" + phpfn + "?" + request;
        String ret = fetchURL(loc);
        System.out.println("HTTP GET " + loc+ " returned: \n" + ret+"\n");

        // Parse the return value for possible JSON data.
        parseReturnValue(ret);        



        // Example 2: Calling an API function which uses HTTP POST ===
        // *** Replace these with the date & code of an uploaded document ***
        String date = "2009-01-07"; 
        String code = "Arr561N9"; 

        phpfn = "updateDocumentMeta.php";        
        request = signRequest(phpfn, apiuser, apikey, annotateuser, 0);
        loc = annotateServer + "/php/" + phpfn + "?" + request;
              

        // Document date and code are added to the request URL
        loc += "&d="+date+"&c="+code;

        System.out.println( "\n\nExample 2 - HTTP POST to URL: " + loc );

        // --- now package up the POST parameters:
        HashMap params = new HashMap();
        params.put("title", "This is the new document title");
        String postParams = urlencodeMap(params);
        
        // POST the settings to the given location
        ret = postURL(loc, postParams);
        System.out.println("\nPOST params: " + postParams +"\n\n");

        // Parse the return value for possible JSON data.
        parseReturnValue(ret);  

    }
    
}